@@ -45,6 +45,11 @@
  *
  * The driver was finished before Intel got the NDA out of the closet.
  * I still don't have the docs.
+ *
+ *
+ * Datasheet is now published and available from 
+ * ftp://download.intel.com/design/network/manuals/8255X_OpenSDM.pdf
+ *    - Michael Brown
  * */
 
 /* Philosophy of this driver.
@@ -105,13 +110,6 @@
 
 static int ioaddr;
 
-typedef unsigned char  u8;
-typedef   signed char  s8;
-typedef unsigned short u16;
-typedef   signed short s16;
-typedef unsigned int   u32;
-typedef   signed int   s32;
-
 enum speedo_offsets {
   SCBStatus = 0, SCBCmd = 2,      /* Rx/Command Unit command and status. */
   SCBPointer = 4,                 /* General purpose pointer. */
@@ -367,16 +365,22 @@ static inline void whereami (const char *str)
 #define whereami(s)
 #endif
 
-static void eepro100_irq(struct nic *nic __unused, irq_action_t action __unused)
+static void eepro100_irq(struct nic *nic __unused, irq_action_t action)
 {
-  switch ( action ) {
-  case DISABLE :
-    break;
-  case ENABLE :
-    break;
-  case FORCE :
-    break;
-  }
+	uint16_t enabled_mask = ( SCBMaskCmdDone | SCBMaskCmdIdle |
+				  SCBMaskEarlyRx | SCBMaskFlowCtl );
+
+	switch ( action ) {
+	case DISABLE :
+		outw(SCBMaskAll, ioaddr + SCBCmd);
+		break;
+	case ENABLE :
+		outw(enabled_mask, ioaddr + SCBCmd);
+		break;
+	case FORCE :
+		outw(enabled_mask | SCBTriggerIntr, ioaddr + SCBCmd);
+		break;
+	}
 }
 
 /* function: eepro100_transmit
@@ -431,7 +435,7 @@ static void eepro100_transmit(struct nic *nic, const char *d, unsigned int t, un
 #endif
 
 	outl(virt_to_bus(&txfd), ioaddr + SCBPointer);
-	outw(INT_MASK | CU_START, ioaddr + SCBCmd);
+	outb(CU_START, ioaddr + SCBCmd);
 	wait_for_cmd_done(ioaddr + SCBCmd);
 
 	s1 = inw (ioaddr + SCBStatus);
@@ -520,6 +524,10 @@ static int eepro100_poll(struct nic *nic, int retrieve)
 #ifdef	DEBUG
 		hd (nic->packet, 0x30);
 #endif
+
+		/* Acknowledge all conceivable interrupts */
+		outw(0xff00, ioaddr + SCBStatus);
+
 		return 1;
 	}
 
@@ -607,8 +615,7 @@ static int eepro100_probe(struct dev *dev, struct pci_device *p)
 	adjust_pci_device(p);
 
 	/* Copy IRQ from PCI information */
-	/* nic->irqno = pci->irq; */
-	nic->irqno = 0;
+	nic->irqno = p->irq;
 
 	if ((do_eeprom_cmd(EE_READ_CMD << 24, 27) & 0xffe0000)
 		== 0xffe0000) {
@@ -638,14 +645,14 @@ static int eepro100_probe(struct dev *dev, struct pci_device *p)
 	udelay (10000);
 	whereami ("Got eeprom.");
 
-	/* Base = 0 */
+	/* Base = 0, disable all interrupts  */
 	outl(0, ioaddr + SCBPointer);
 	outw(INT_MASK | RX_ADDR_LOAD, ioaddr + SCBCmd);
 	wait_for_cmd_done(ioaddr + SCBCmd);
 	whereami ("set rx base addr.");
 
 	outl(virt_to_bus(&lstats), ioaddr + SCBPointer);
-	outw(INT_MASK | CU_STATSADDR, ioaddr + SCBCmd);
+	outb(CU_STATSADDR, ioaddr + SCBCmd);
 	wait_for_cmd_done(ioaddr + SCBCmd);
 	whereami ("set stats addr.");
 
@@ -664,7 +671,7 @@ static int eepro100_probe(struct dev *dev, struct pci_device *p)
 	rxfds[RXFD_COUNT-1].link    = virt_to_bus(&rxfds[0]);
 
 	outl(virt_to_bus(&rxfds[0]), ioaddr + SCBPointer);
-	outw(INT_MASK | RX_START, ioaddr + SCBCmd);
+	outb(RX_START, ioaddr + SCBCmd);
 	wait_for_cmd_done(ioaddr + SCBCmd);
 
 	whereami ("started RX process.");
@@ -673,7 +680,7 @@ static int eepro100_probe(struct dev *dev, struct pci_device *p)
 
 	/* Base = 0 */
 	outl(0, ioaddr + SCBPointer);
-	outw(INT_MASK | CU_CMD_BASE, ioaddr + SCBCmd);
+	outb(CU_CMD_BASE, ioaddr + SCBCmd);
 	wait_for_cmd_done(ioaddr + SCBCmd);
 
 	whereami ("set TX base addr.");
@@ -732,7 +739,7 @@ static int eepro100_probe(struct dev *dev, struct pci_device *p)
 	confcmd.data[21] = (rx_mode & 1) ? 0x0D: 0x05;
 
 	outl(virt_to_bus(&txfd), ioaddr + SCBPointer);
-	outw(INT_MASK | CU_START, ioaddr + SCBCmd);
+	outb(CU_START, ioaddr + SCBCmd);
 	wait_for_cmd_done(ioaddr + SCBCmd);
 
 	whereami ("started TX thingy (config, iasetup).");
