@@ -402,7 +402,7 @@ static int pcnet32_init_ring(struct nic *nic)
 	lp->cur_rx = lp->cur_tx = 0;
 
 	for (i = 0; i < RX_RING_SIZE; i++) {
-		rx_ring[i].base = (u32) virt_to_le32desc(&rxb[i]);
+		rx_ring[i].base = (u32) virt_to_le32desc(&rxb[i * PKT_BUF_SZ]);
 		rx_ring[i].buf_length = le16_to_cpu(-PKT_BUF_SZ);
 		rx_ring[i].status = le16_to_cpu(0x8000);
 	}
@@ -444,6 +444,7 @@ static void pcnet32_reset(struct nic *nic)
 	if (lp->options & PCNET32_PORT_ASEL)
 		val |= 2;
 	lp->a.write_bcr(ioaddr, 2, val);
+
 	/* handle full duplex setting */
 	if (lp->full_duplex) {
 		val = lp->a.read_bcr(ioaddr, 9) & ~3;
@@ -565,7 +566,7 @@ static int pcnet32_poll(struct nic *nic __unused, int retrieve)
 	if (status == 0x03) {
 		nic->packetlen =
 		    (le32_to_cpu(rx_ring[entry].msg_length) & 0xfff) - 4;
-		memcpy(nic->packet, &rxb[entry], nic->packetlen);
+		memcpy(nic->packet, &rxb[entry * PKT_BUF_SZ], nic->packetlen);
 
 		/* Andrew Boyd of QNX reports that some revs of the 79C765
 		 * clear the buffer length */
@@ -646,10 +647,10 @@ static void pcnet32_disable(struct dev *dev __unused)
 	lp->a.write_csr(ioaddr, 0, 0x0004);
 
 	/*
-	 * Switch back to 16-bit mode to avoid problesm with dumb 
+	 * Switch back to 16-bit mode to avoid problems with dumb 
 	 * DOS packet driver after a warm reboot
 	 */
-	lp->a.write_bcr(ioaddr, 20, 4);
+	lp->a.write_bcr(ioaddr, 20, 0);
 }
 
 /**************************************************************************
@@ -712,7 +713,7 @@ static int pcnet32_probe(struct dev *dev, struct pci_device *pci)
 	chip_version =
 	    a->read_csr(ioaddr, 88) | (a->read_csr(ioaddr, 89) << 16);
 
-	dprintf(("PCnet chip version is %0xhX\n", chip_version));
+	dprintf(("PCnet chip version is 0x%X\n", chip_version));
 	if ((chip_version & 0xfff) != 0x003)
 		return 0;
 
@@ -774,6 +775,7 @@ static int pcnet32_probe(struct dev *dev, struct pci_device *pci)
 		mii = 1;
 		break;
 	default:
+		chipname = "UNKNOWN";
 		printf("PCnet version %#x, no PCnet32 chip.\n",
 		       chip_version);
 		return 0;
@@ -806,7 +808,7 @@ static int pcnet32_probe(struct dev *dev, struct pci_device *pci)
 		nic->node_addr[i] = promaddr[i];
 	}
 	/* Print out some hardware info */
-	printf("%s: %! at ioaddr %hX, ", pci->name, nic->node_addr,
+	printf("%s: %! at ioaddr 0x%hX, ", chipname, nic->node_addr,
 	       ioaddr);
 
 	/* Set to pci bus master */
@@ -893,7 +895,6 @@ static int pcnet32_probe(struct dev *dev, struct pci_device *pci)
 	/* switch pcnet32 to 32bit mode */
 	a->write_bcr(ioaddr, 20, 2);
 
-
 	a->write_csr(ioaddr, 1, (virt_to_bus(&lp->init_block)) & 0xffff);
 	a->write_csr(ioaddr, 2, (virt_to_bus(&lp->init_block)) >> 16);
 
@@ -912,8 +913,8 @@ static int pcnet32_probe(struct dev *dev, struct pci_device *pci)
 
 	/* point to NIC specific routines */
 	pcnet32_reset(nic);
-	if (1) {
-	        int tmp;
+	if (mii) {
+		int tmp;
 		int phy, phy_idx = 0;
 		u16 mii_lpa;
 		lp->phys[0] = 1;	/* Default Setting */
@@ -949,7 +950,14 @@ static int pcnet32_probe(struct dev *dev, struct pci_device *pci)
 		else if (lp->mii_if.advertising & ADVERTISE_10HALF)
 			printf("10Mbps Half-Duplex\n");
 		else
-			printf("\n");
+			printf("No Link?\n");
+	} else {
+		/* The older chips are fixed 10Mbps, and some support full duplex,
+		 * although not via autonegotiation, but only via configuration.  */
+		if (fdx)
+			printf("10Mbps Full-Duplex\n");
+		else
+			printf("10Mbps Half-Duplex\n");
 	}
 
 	nic->poll     = pcnet32_poll;
@@ -996,9 +1004,9 @@ static void mdio_write(struct nic *nic __unused, int phy_id, int reg_num,
 #endif
 
 static struct pci_id pcnet32_nics[] = {
-	PCI_ROM(0x1022, 0x2000, "lancepci", "AMD Lance/PCI"),
-	PCI_ROM(0x1022, 0x2625, "pcnetfastiii", "AMD Lance/PCI PCNet/32"),
-	PCI_ROM(0x1022, 0x2001, "amdhomepna", "AMD Lance/HomePNA"),
+	PCI_ROM(0x1022, 0x2000, "pcnet32", "AMD PCnet/PCI"),
+	PCI_ROM(0x1022, 0x2625, "pcnetfastiii", "AMD PCNet FAST III"),
+	PCI_ROM(0x1022, 0x2001, "amdhomepna", "AMD PCnet/HomePNA"),
 };
 
 static struct pci_driver pcnet32_driver __pci_driver = {
