@@ -67,6 +67,7 @@ static BufferDesc rxd[NUM_RX_DESC];
 static unsigned char txb[TX_BUF_SIZE];
 static unsigned char rxb[NUM_RX_DESC * RX_BUF_SIZE];
 
+#if 0
 static struct mac_chip_info {
     const char *name;
     u16 vendor_id, device_id, flags;
@@ -78,6 +79,7 @@ static struct mac_chip_info {
       PCI_COMMAND_IO|PCI_COMMAND_MASTER, SIS900_TOTAL_SIZE},
     {0,0,0,0,0} /* 0 terminated list. */
 };
+#endif
 
 static void sis900_read_mode(struct nic *nic, int phy_addr, int *speed, int *duplex);
 static void amd79c901_read_mode(struct nic *nic, int phy_addr, int *speed, int *duplex);
@@ -93,10 +95,11 @@ static struct mii_chip_info {
 } mii_chip_table[] = {
     {"SiS 900 Internal MII PHY", 0x001d, 0x8000, sis900_read_mode},
     {"SiS 7014 Physical Layer Solution", 0x0016, 0xf830,sis900_read_mode},
-    {"AMD 79C901 10BASE-T PHY",  0x0000, 0x35b9, amd79c901_read_mode},
-    {"AMD 79C901 HomePNA PHY",   0x0000, 0x35c8, amd79c901_read_mode},
-    {"ICS 1893 Integrated PHYceiver"   , 0x0015, 0xf441,ics1893_read_mode},
-    {"RTL 8201 10/100Mbps Phyceiver"   , 0x0000, 0x8201,rtl8201_read_mode},
+    {"AMD 79C901 10BASE-T PHY",  0x0000, 0x6B70, amd79c901_read_mode},
+    {"AMD 79C901 HomePNA PHY",   0x0000, 0x6B90, amd79c901_read_mode},
+    {"ICS 1893 Integrated PHYceiver"   , 0x0015, 0xf440,ics1893_read_mode},
+//  {"NS 83851 PHY",0x2000, 0x5C20, MIX },
+    {"RTL 8201 10/100Mbps Phyceiver"   , 0x0000, 0x8200,rtl8201_read_mode},
     {"VIA 6103 10/100Mbps Phyceiver", 0x0101, 0x8f20,vt6103_read_mode},
     {0,0,0,0}
 };
@@ -132,8 +135,9 @@ static u16  sis900_read_eeprom(int location);
 static void sis900_mdio_reset(long mdio_addr);
 static void sis900_mdio_idle(long mdio_addr);
 static u16  sis900_mdio_read(int phy_id, int location);
+#if 0
 static void sis900_mdio_write(int phy_id, int location, int val);
-
+#endif
 static void sis900_init(struct nic *nic);
 
 static void sis900_reset(struct nic *nic);
@@ -146,10 +150,12 @@ static void sis900_check_mode(struct nic *nic);
 
 static void sis900_transmit(struct nic *nic, const char *d, 
                             unsigned int t, unsigned int s, const char *p);
-static int  sis900_poll(struct nic *nic);
+static int  sis900_poll(struct nic *nic, int retrieve);
 
 static void sis900_disable(struct dev *dev);
 
+static void sis900_irq(struct nic *nic, irq_action_t action);
+
 /**
  *	sis900_get_mac_addr: - Get MAC address for stand alone SiS900 model
  *	@pci_dev: the sis900 pci device
@@ -193,7 +199,7 @@ static int sis900_get_mac_addr(struct pci_device * pci_dev __unused, struct nic
  *	MAC address is read into @net_dev->dev_addr.
  */
 
-static int sis96x_get_mac_addr(struct pci_device * pci_dev, struct nic *nic)
+static int sis96x_get_mac_addr(struct pci_device * pci_dev __unused, struct nic *nic)
 {
 /* 	long ioaddr = net_dev->base_addr; */
 	long ee_addr = ioaddr + mear;
@@ -322,6 +328,8 @@ static int sis900_probe(struct dev *dev, struct pci_device *pci)
     if (pci->ioaddr == 0)
         return 0;
 
+    nic->irqno  = 0;
+    nic->ioaddr = pci->ioaddr & ~3;
     ioaddr  = pci->ioaddr & ~3;
     vendor  = pci->vendor;
     dev_id  = pci->dev_id;
@@ -380,7 +388,8 @@ static int sis900_probe(struct dev *dev, struct pci_device *pci)
         /* search our mii table for the current mii */ 
         for (i = 0; mii_chip_table[i].phy_id1; i++) {
 
-            if (phy_id0 == mii_chip_table[i].phy_id0) {
+            if ((phy_id0 == mii_chip_table[i].phy_id0) &&
+                ((phy_id1 & 0xFFF0) == mii_chip_table[i].phy_id1)){
 
                 printf("sis900_probe: %s transceiver found at address %d.\n",
                        mii_chip_table[i].name, phy_addr);
@@ -411,6 +420,7 @@ static int sis900_probe(struct dev *dev, struct pci_device *pci)
     dev->disable  = sis900_disable;
     nic->poll     = sis900_poll;
     nic->transmit = sis900_transmit;
+    nic->irq      = sis900_irq;
 
     return 1;
 }
@@ -447,7 +457,7 @@ static u16 sis900_read_eeprom(int location)
 
     outl(0, ee_addr);
     eeprom_delay();
-    outl(EECLK, ee_addr);
+    outl(EECS, ee_addr);
     eeprom_delay();
 
     /* Shift the read command (9) bits out. */
@@ -458,7 +468,7 @@ static u16 sis900_read_eeprom(int location)
         outl(dataval | EECLK, ee_addr);
         eeprom_delay();
     }
-    outb(EECS, ee_addr);
+    outl(EECS, ee_addr);
     eeprom_delay();
 
     /* read the 16-bits data in */
@@ -474,7 +484,7 @@ static u16 sis900_read_eeprom(int location)
     /* Terminate the EEPROM access. */
     outl(0, ee_addr);
     eeprom_delay();
-    outl(EECLK, ee_addr);
+//  outl(EECLK, ee_addr);
 
     return (retval);
 }
@@ -539,6 +549,7 @@ static u16 sis900_mdio_read(int phy_id, int location)
     return retval;
 }
 
+#if 0
 static void sis900_mdio_write(int phy_id, int location, int value)
 {
     long mdio_addr = ioaddr + mear;
@@ -578,6 +589,7 @@ static void sis900_mdio_write(int phy_id, int location, int value)
     outl(0x00, mdio_addr);
     return;
 }
+#endif
 
 
 /* Function: sis900_init
@@ -636,7 +648,7 @@ sis900_reset(struct nic *nic __unused)
         status ^= (inl(isr + ioaddr) & status);
     }
 
-    if( (pci_revision == SIS635A_900_REV) || (pci_revision == SIS900B_900_REV) )
+    if( (pci_revision >= SIS635A_900_REV) || (pci_revision == SIS900B_900_REV) )
             outl(PESEL | RND_CNT, ioaddr + cfg);
     else
             outl(PESEL, ioaddr + cfg);
@@ -1081,7 +1093,7 @@ sis900_transmit(struct nic  *nic,
                 const char  *p)     /* Packet */
 {
     u32 to, nstype;
-    u32 tx_status;
+    volatile u32 tx_status;
     
     /* Stop the transmitter */
     outl(TxDIS | inl(ioaddr + cr), ioaddr + cr);
@@ -1120,7 +1132,7 @@ sis900_transmit(struct nic  *nic,
 
     to = currticks() + TX_TIMEOUT;
 
-    while ((((volatile u32) tx_status=txd.cmdsts) & OWN) && (currticks() < to))
+    while (((tx_status=txd.cmdsts) & OWN) && (currticks() < to))
         /* wait */ ;
 
     if (currticks() >= to) {
@@ -1151,7 +1163,7 @@ sis900_transmit(struct nic  *nic,
  */
 
 static int
-sis900_poll(struct nic *nic)
+sis900_poll(struct nic *nic, int retrieve)
 {
     u32 rx_status = rxd[cur_rx].cmdsts;
     int retstat = 0;
@@ -1166,6 +1178,8 @@ sis900_poll(struct nic *nic)
         printf("sis900_poll: got a packet: cur_rx:%d, status:%X\n",
                cur_rx, rx_status);
 
+    if ( ! retrieve ) return 1;
+    
     nic->packetlen = (rx_status & DSIZE) - CRC_SIZE;
 
     if (rx_status & (ABORT|OVERRUN|TOOLONG|RUNT|RXISERR|CRCERR|FAERR)) {
@@ -1218,6 +1232,30 @@ sis900_disable(struct dev *dev)
     outl(RxDIS | TxDIS | inl(ioaddr + cr), ioaddr + cr);
 }
 
+
+/* Function: sis900_irq
+ *
+ * Description: Enable, Disable, or Force, interrupts
+ *    
+ * Arguments: struct nic *nic:          NIC data structure
+ *            irq_action_t action:      Requested action       
+ *
+ * Returns:   void.
+ */
+
+static void
+sis900_irq(struct nic *nic __unused, irq_action_t action __unused)
+{
+  switch ( action ) {
+  case DISABLE :
+    break;
+  case ENABLE :
+    break;
+  case FORCE :
+    break;
+  }
+}
+
 static struct pci_id sis900_nics[] = {
 PCI_ROM(0x1039, 0x0900, "sis900",  "SIS900"),
 PCI_ROM(0x1039, 0x7016, "sis7016", "SIS7016"),
