@@ -52,13 +52,6 @@
 #define drv_version "v1.12"
 #define drv_date "2004-03-21"
 
-typedef unsigned char u8;
-typedef signed char s8;
-typedef unsigned short u16;
-typedef signed short s16;
-typedef unsigned int u32;
-typedef signed int s32;
-
 #define HZ 100
 
 /* Condensed operations for readability. */
